@@ -100,9 +100,16 @@ where
     T: Clone + Ord,
 {
     pub fn new(n: usize, epsilon: f64) -> Self {
+        if n == 0 {
+            panic!("n should be greater than 0")
+        }
+
+        if epsilon <= 0.0 {
+            panic!("epsilon should be greater than 0");
+        }
+
         // number_of_leves = ceil(log2(n))
         // block_size = floor(log2(epsilon * N) / epsilon)
-
         let epsilon_n: f64 = (n as f64) * epsilon;
         let number_of_levels = if epsilon_n > 1.0 {
             (epsilon_n as f64).log2().ceil() as usize
@@ -178,8 +185,11 @@ where
 
         let rank: i64 = ((self.cnt as f64) * r).floor() as i64;
         let epsilon_n: i64 = ((self.cnt as f64) * self.epsilon).floor() as i64;
-        let e = find_idx(&self.cached_s_m.as_ref().unwrap(), rank, epsilon_n).unwrap();
-        return e;
+        if let Some(e) = find_idx(&self.cached_s_m.as_ref().unwrap(), rank, epsilon_n) {
+            e
+        } else {
+            self.cached_s_m.as_ref().unwrap().last().unwrap().val.clone()
+        }
     }
 
     fn calc_s_m(&mut self, epsilon: f64) -> Vec<RankInfo<T>> {
@@ -388,6 +398,10 @@ where
     T: Clone + Ord + std::fmt::Debug,
 {
     pub fn new(epsilon: f64) -> Self {
+        if epsilon <= 0.0 {
+            panic!("epsilon should be greater than 0");
+        }
+
         let s = vec![];
 
         let n = (1.0_f64 / epsilon).floor() as usize;
@@ -439,8 +453,11 @@ where
 
         let rank: i64 = ((self.cnt as f64) * r).floor() as i64;
         let epsilon_n: i64 = ((self.cnt as f64) * self.epsilon).floor() as i64;
-        let e = find_idx(&self.cached_s_m.as_ref().unwrap(), rank, epsilon_n).unwrap();
-        return e;
+        if let Some(e) = find_idx(&self.cached_s_m.as_ref().unwrap(), rank, epsilon_n) {
+            e
+        } else {
+            self.cached_s_m.as_ref().unwrap().last().unwrap().val.clone()
+        }
     }
 
     #[inline]
@@ -489,6 +506,32 @@ mod tests {
         assert_eq!(compressed_vals, vec![1, 4, 7, 12, 17]);
     }
 
+    #[test]
+    #[should_panic]
+    fn test_panic_on_negative_epsilon_when_constructing_fixedsize_summary() {
+        let _ = FixedSizeEpsilonSummary::<usize>::new(100, -0.1);
+    }
+
+    #[test]
+    #[should_panic]
+    fn test_panic_on_negative_epsilon_when_constructing_unbound_summary() {
+        let _ = UnboundEpsilonSummary::<usize>::new(-0.1);
+    }
+
+    #[test]
+    fn test_query_unbound_summary_with_insufficient_values() {
+        let epsilon = 0.1;
+        let n = 3;
+        let mut s = FixedSizeEpsilonSummary::new(n, epsilon);
+        for i in 1..=n {
+            s.update(i);
+        }
+
+        let rank: f64 = 1.0;
+        let ans = s.query(rank);
+        assert!(n == ans);
+    }
+
     #[test]
     fn test_query_with_small_n_on_fixedsize_summary() {
         let epsilon = 0.1;
